searchData={"items":[{"type":"module","title":"CommonTwos","doc":"A utility module for finding the greatest common divisor (GCD) of two numbers\nby counting trailing zeros in their binary representation.\n\nThis module provides efficient algorithms for finding common factors between\ntwo integers using bitwise operations. The primary function `of/2` returns\na tuple containing the shift count and the reduced values after removing\ncommon factors of 2.","ref":"CommonTwos.html"},{"type":"module","title":"Examples - CommonTwos","doc":"iex> CommonTwos.of(12, 8)\n    {2, 3, 2}\n\n    iex> CommonTwos.of(16, 24)\n    {3, 2, 3}\n\n    iex> CommonTwos.of(7, 11)\n    {0, 7, 11}","ref":"CommonTwos.html#module-examples"},{"type":"module","title":"Algorithm - CommonTwos","doc":"The algorithm works by:\n1. Counting how many trailing zeros both numbers have in common\n2. Shifting both numbers right by that amount\n3. Returning the shift count and the reduced numbers\n\nThis is particularly useful for:\n- Finding common factors efficiently\n- Optimizing mathematical operations\n- Number theory applications","ref":"CommonTwos.html#module-algorithm"},{"type":"function","title":"CommonTwos.of/2","doc":"Finds the common factors of 2 between two numbers and returns the shift count\nalong with the reduced values.","ref":"CommonTwos.html#of/2"},{"type":"function","title":"Parameters - CommonTwos.of/2","doc":"- `a` - First non-negative integer\n  - `b` - Second non-negative integer","ref":"CommonTwos.html#of/2-parameters"},{"type":"function","title":"Returns - CommonTwos.of/2","doc":"A tuple `{shift, reduced_a, reduced_b}` where:\n  - `shift` - The number of common factors of 2 (trailing zeros in binary)\n  - `reduced_a` - The value of `a` after removing common factors of 2\n  - `reduced_b` - The value of `b` after removing common factors of 2","ref":"CommonTwos.html#of/2-returns"},{"type":"function","title":"Examples - CommonTwos.of/2","doc":"iex> CommonTwos.of(12, 8)\n    {2, 3, 2}\n\n    iex> CommonTwos.of(16, 24)\n    {3, 2, 3}\n\n    iex> CommonTwos.of(7, 11)\n    {0, 7, 11}\n\n    iex> CommonTwos.of(0, 8)\n    {3, 0, 1}","ref":"CommonTwos.html#of/2-examples"},{"type":"function","title":"Notes - CommonTwos.of/2","doc":"- If both numbers are 0, the result will be `{0, 0, 0}`\n- If one number is 0, the shift count will be based on the non-zero number\n- The algorithm uses bitwise operations for efficiency","ref":"CommonTwos.html#of/2-notes"},{"type":"extras","title":"CommonTwos","doc":"# CommonTwos\n\nA utility library for finding common factors of 2 between two numbers using efficient bitwise operations.","ref":"readme.html"},{"type":"extras","title":"Overview - CommonTwos","doc":"CommonTwos provides algorithms for finding the greatest common divisor (GCD) by counting trailing zeros in binary representation. This is particularly useful for mathematical optimizations and number theory applications.","ref":"readme.html#overview"},{"type":"extras","title":"Features - CommonTwos","doc":"- **Efficient bitwise operations** for finding common factors\n- **Type-safe** with comprehensive specs\n- **Well-documented** with examples and use cases\n- **Production-ready** with full development tooling","ref":"readme.html#features"},{"type":"extras","title":"Installation - CommonTwos","doc":"The package can be installed by adding `common_twos` to your list of dependencies in `mix.exs`:\n\n```elixir\ndef deps do\n  [\n    {:common_twos, \"~> 1.0\"}\n  ]\nend\n```","ref":"readme.html#installation"},{"type":"extras","title":"Usage - CommonTwos","doc":"","ref":"readme.html#usage"},{"type":"extras","title":"Basic Usage - CommonTwos","doc":"```elixir\n# Find common factors of 2 between two numbers\n{shift, reduced_a, reduced_b} = CommonTwos.of(12, 8)\n# Returns: {2, 3, 2}\n\n# The shift count represents common factors of 2\n# The reduced values are the numbers after removing those factors\n```","ref":"readme.html#basic-usage"},{"type":"extras","title":"Examples - CommonTwos","doc":"```elixir\n# Numbers with common factors of 2\nCommonTwos.of(16, 24)  # Returns: {3, 2, 3}\nCommonTwos.of(8, 32)   # Returns: {3, 1, 4}\n\n# Numbers without common factors of 2\nCommonTwos.of(7, 11)   # Returns: {0, 7, 11}\nCommonTwos.of(3, 5)    # Returns: {0, 3, 5}\n\n# Edge cases\nCommonTwos.of(0, 8)    # Returns: {3, 0, 1}\nCommonTwos.of(0, 0)    # Returns: {0, 0, 0}\n```","ref":"readme.html#examples"},{"type":"extras","title":"Algorithm - CommonTwos","doc":"The algorithm works by:\n\n1. **Counting trailing zeros**: Both numbers are shifted right until at least one has a 1 in the least significant bit\n2. **Tracking shifts**: The number of shifts performed is the count of common factors of 2\n3. **Returning results**: A tuple containing the shift count and the reduced values\n\nThis is particularly efficient because:\n- Uses bitwise operations (`bor`, `band`, `bsr`)\n- Avoids division operations\n- Works well with binary number representations","ref":"readme.html#algorithm"},{"type":"extras","title":"API Reference - CommonTwos","doc":"### `CommonTwos.of/2`\n\n```elixir\n@spec of(non_neg_integer(), non_neg_integer()) ::\n        {non_neg_integer(), non_neg_integer(), non_neg_integer()}\n```\n\nFinds the common factors of 2 between two numbers.\n\n**Parameters:**\n- `a` - First non-negative integer\n- `b` - Second non-negative integer\n\n**Returns:**\nA tuple `{shift, reduced_a, reduced_b}` where:\n- `shift` - The number of common factors of 2 (trailing zeros in binary)\n- `reduced_a` - The value of `a` after removing common factors of 2\n- `reduced_b` - The value of `b` after removing common factors of 2","ref":"readme.html#api-reference"},{"type":"extras","title":"License - CommonTwos","doc":"Copyright (c) 2025 University of Kitakyushu\n\nLicensed under the Apache License, Version 2.0 (the \"License\");\nyou may not use this file except in compliance with the License.\nYou may obtain a copy of the License at [http://www.apache.org/licenses/LICENSE-2.0](http://www.apache.org/licenses/LICENSE-2.0)\n\nUnless required by applicable law or agreed to in writing, software\ndistributed under the License is distributed on an \"AS IS\" BASIS,\nWITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.\nSee the License for the specific language governing permissions and\nlimitations under the License.","ref":"readme.html#license"},{"type":"extras","title":"LICENSE","doc":"Apache License\n                           Version 2.0, January 2004\n                        http://www.apache.org/licenses/\n\n   TERMS AND CONDITIONS FOR USE, REPRODUCTION, AND DISTRIBUTION\n\n   1. Definitions.\n\n      \"License\" shall mean the terms and conditions for use, reproduction,\n      and distribution as defined by Sections 1 through 9 of this document.\n\n      \"Licensor\" shall mean the copyright owner or entity authorized by\n      the copyright owner that is granting the License.\n\n      \"Legal Entity\" shall mean the union of the acting entity and all\n      other entities that control, are controlled by, or are under common\n      control with that entity. For the purposes of this definition,\n      \"control\" means (i) the power, direct or indirect, to cause the\n      direction or management of such entity, whether by contract or\n      otherwise, or (ii) ownership of fifty percent (50%) or more of the\n      outstanding shares, or (iii) beneficial ownership of such entity.\n\n      \"You\" (or \"Your\") shall mean an individual or Legal Entity\n      exercising permissions granted by this License.\n\n      \"Source\" form shall mean the preferred form for making modifications,\n      including but not limited to software source code, documentation\n      source, and configuration files.\n\n      \"Object\" form shall mean any form resulting from mechanical\n      transformation or translation of a Source form, including but\n      not limited to compiled object code, generated documentation,\n      and conversions to other media types.\n\n      \"Work\" shall mean the work of authorship, whether in Source or\n      Object form, made available under the License, as indicated by a\n      copyright notice that is included in or attached to the work\n      (an example is provided in the Appendix below).\n\n      \"Derivative Works\" shall mean any work, whether in Source or Object\n      form, that is based on (or derived from) the Work and for which the\n      editorial revisions, annotations, elaborations, or other modifications\n      represent, as a whole, an original work of authorship. For the purposes\n      of this License, Derivative Works shall not include works that remain\n      separable from, or merely link (or bind by name) to the interfaces of,\n      the Work and Derivative Works thereof.\n\n      \"Contribution\" shall mean any work of authorship, including\n      the original version of the Work and any modifications or additions\n      to that Work or Derivative Works thereof, that is intentionally\n      submitted to Licensor for inclusion in the Work by the copyright owner\n      or by an individual or Legal Entity authorized to submit on behalf of\n      the copyright owner. For the purposes of this definition, \"submitted\"\n      means any form of electronic, verbal, or written communication sent\n      to the Licensor or its representatives, including but not limited to\n      communication on electronic mailing lists, source code control systems,\n      and issue tracking systems that are managed by, or on behalf of, the\n      Licensor for the purpose of discussing and improving the Work, but\n      excluding communication that is conspicuously marked or otherwise\n      designated in writing by the copyright owner as \"Not a Contribution.\"\n\n      \"Contributor\" shall mean Licensor and any individual or Legal Entity\n      on behalf of whom a Contribution has been received by Licensor and\n      subsequently incorporated within the Work.\n\n   2. Grant of Copyright License. Subject to the terms and conditions of\n      this License, each Contributor hereby grants to You a perpetual,\n      worldwide, non-exclusive, no-charge, royalty-free, irrevocable\n      copyright license to reproduce, prepare Derivative Works of,\n      publicly display, publicly perform, sublicense, and distribute the\n      Work and such Derivative Works in Source or Object form.\n\n   3. Grant of Patent License. Subject to the terms and conditions of\n      this License, each Contributor hereby grants to You a perpetual,\n      worldwide, non-exclusive, no-charge, royalty-free, irrevocable\n      (except as stated in this section) patent license to make, have made,\n      use, offer to sell, sell, import, and otherwise transfer the Work,\n      where such license applies only to those patent claims licensable\n      by such Contributor that are necessarily infringed by their\n      Contribution(s) alone or by combination of their Contribution(s)\n      with the Work to which such Contribution(s) was submitted. If You\n      institute patent litigation against any entity (including a\n      cross-claim or counterclaim in a lawsuit) alleging that the Work\n      or a Contribution incorporated within the Work constitutes direct\n      or contributory patent infringement, then any patent licenses\n      granted to You under this License for that Work shall terminate\n      as of the date such litigation is filed.\n\n   4. Redistribution. You may reproduce and distribute copies of the\n      Work or Derivative Works thereof in any medium, with or without\n      modifications, and in Source or Object form, provided that You\n      meet the following conditions:\n\n      (a) You must give any other recipients of the Work or\n          Derivative Works a copy of this License; and\n\n      (b) You must cause any modified files to carry prominent notices\n          stating that You changed the files; and\n\n      (c) You must retain, in the Source form of any Derivative Works\n          that You distribute, all copyright, patent, trademark, and\n          attribution notices from the Source form of the Work,\n          excluding those notices that do not pertain to any part of\n          the Derivative Works; and\n\n      (d) If the Work includes a \"NOTICE\" text file as part of its\n          distribution, then any Derivative Works that You distribute must\n          include a readable copy of the attribution notices contained\n          within such NOTICE file, excluding those notices that do not\n          pertain to any part of the Derivative Works, in at least one\n          of the following places: within a NOTICE text file distributed\n          as part of the Derivative Works; within the Source form or\n          documentation, if provided along with the Derivative Works; or,\n          within a display generated by the Derivative Works, if and\n          wherever such third-party notices normally appear. The contents\n          of the NOTICE file are for informational purposes only and\n          do not modify the License. You may add Your own attribution\n          notices within Derivative Works that You distribute, alongside\n          or as an addendum to the NOTICE text from the Work, provided\n          that such additional attribution notices cannot be construed\n          as modifying the License.\n\n      You may add Your own copyright statement to Your modifications and\n      may provide additional or different license terms and conditions\n      for use, reproduction, or distribution of Your modifications, or\n      for any such Derivative Works as a whole, provided Your use,\n      reproduction, and distribution of the Work otherwise complies with\n      the conditions stated in this License.\n\n   5. Submission of Contributions. Unless You explicitly state otherwise,\n      any Contribution intentionally submitted for inclusion in the Work\n      by You to the Licensor shall be under the terms and conditions of\n      this License, without any additional terms or conditions.\n      Notwithstanding the above, nothing herein shall supersede or modify\n      the terms of any separate license agreement you may have executed\n      with Licensor regarding such Contributions.\n\n   6. Trademarks. This License does not grant permission to use the trade\n      names, trademarks, service marks, or product names of the Licensor,\n      except as required for reasonable and customary use in describing the\n      origin of the Work and reproducing the content of the NOTICE file.\n\n   7. Disclaimer of Warranty. Unless required by applicable law or\n      agreed to in writing, Licensor provides the Work (and each\n      Contributor provides its Contributions) on an \"AS IS\" BASIS,\n      WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or\n      implied, including, without limitation, any warranties or conditions\n      of TITLE, NON-INFRINGEMENT, MERCHANTABILITY, or FITNESS FOR A\n      PARTICULAR PURPOSE. You are solely responsible for determining the\n      appropriateness of using or redistributing the Work and assume any\n      risks associated with Your exercise of permissions under this License.\n\n   8. Limitation of Liability. In no event and under no legal theory,\n      whether in tort (including negligence), contract, or otherwise,\n      unless required by applicable law (such as deliberate and grossly\n      negligent acts) or agreed to in writing, shall any Contributor be\n      liable to You for damages, including any direct, indirect, special,\n      incidental, or consequential damages of any character arising as a\n      result of this License or out of the use or inability to use the\n      Work (including but not limited to damages for loss of goodwill,\n      work stoppage, computer failure or malfunction, or any and all\n      other commercial damages or losses), even if such Contributor\n      has been advised of the possibility of such damages.\n\n   9. Accepting Warranty or Additional Liability. While redistributing\n      the Work or Derivative Works thereof, You may choose to offer,\n      and charge a fee for, acceptance of support, warranty, indemnity,\n      or other liability obligations and/or rights consistent with this\n      License. However, in accepting such obligations, You may act only\n      on Your own behalf and on Your sole responsibility, not on behalf\n      of any other Contributor, and only if You agree to indemnify,\n      defend, and hold each Contributor harmless for any liability\n      incurred by, or claims asserted against, such Contributor by reason\n      of your accepting any such warranty or additional liability.\n\n   END OF TERMS AND CONDITIONS\n\n   APPENDIX: How to apply the Apache License to your work.\n\n      To apply the Apache License to your work, attach the following\n      boilerplate notice, with the fields enclosed by brackets \"[]\"\n      replaced with your own identifying information. (Don't include\n      the brackets!)  The text should be enclosed in the appropriate\n      comment syntax for the file format. We also recommend that a\n      file or class name and description of purpose be included on the\n      same \"printed page\" as the copyright notice for easier\n      identification within third-party archives.\n\n   Copyright [yyyy] [name of copyright owner]\n\n   Licensed under the Apache License, Version 2.0 (the \"License\");\n   you may not use this file except in compliance with the License.\n   You may obtain a copy of the License at\n\n       http://www.apache.org/licenses/LICENSE-2.0\n\n   Unless required by applicable law or agreed to in writing, software\n   distributed under the License is distributed on an \"AS IS\" BASIS,\n   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.\n   See the License for the specific language governing permissions and\n   limitations under the License.","ref":"license.html"},{"type":"extras","title":"Changelog","doc":"# Changelog\n\nAll notable changes to this project will be documented in this file.\n\nThe format is based on [Keep a Changelog](https://keepachangelog.com/en/1.0.0/),\nand this project adheres to [Semantic Versioning](https://semver.org/spec/v2.0.0.html).\n\n## [1.0.0] - 2025-07-18","ref":"changelog.html"},{"type":"extras","title":"Added - Changelog","doc":"- Development tooling configuration\n- Initial project setup with development tooling\n- Comprehensive documentation and README\n- Quality assurance tools (Credo, Dialyzer, Spellweaver)\n- **Core Algorithm**: Implemented `CommonTwos.of/2` function for finding common factors of 2\n- **Bitwise Operations**: Efficient algorithm using `Bitwise.bor`, `Bitwise.band`, and `Bitwise.bsr`\n- **Type Safety**: Comprehensive type specs for all functions\n- **Edge Case Handling**: Proper handling of zero values and edge conditions","ref":"changelog.html#added"},{"type":"extras","title":"Documentation - Changelog","doc":"- **Module Documentation**: Complete `@moduledoc` with algorithm explanation and examples\n- **Function Documentation**: Detailed `@doc` for `of/2` with parameters, returns, and examples\n- **README.md**: Comprehensive project documentation with usage examples\n- **API Reference**: Complete documentation of public functions and their behavior","ref":"changelog.html#documentation"},{"type":"extras","title":"Development - Changelog","doc":"- **Project Configuration**: Full development tooling setup\n- **Quality Tools**: Credo for static analysis, Dialyzer for type checking\n- **Documentation Tools**: ExDoc for HTML documentation generation\n- **Spell Checking**: Spellweaver integration with custom word list\n- **Code Standards**: nstandard for formatting and standards enforcement","ref":"changelog.html#development"},{"type":"extras","title":"Infrastructure - Changelog","doc":"- **License**: Apache 2.0 license with University of Kitakyushu copyright\n- **Package Configuration**: Hex.pm package configuration for distribution\n- **Development Aliases**: `mix check` for comprehensive quality assurance\n- **Dependencies**: Production and development dependencies properly configured","ref":"changelog.html#infrastructure"},{"type":"extras","title":"Technical Details - Changelog","doc":"- **Algorithm**: Recursive implementation counting trailing zeros in binary representation\n- **Return Format**: Tuple `{shift, reduced_a, reduced_b}` with shift count and reduced values\n- **Performance**: Efficient bitwise operations avoiding division\n- **Compatibility**: Supports Elixir ~> 1.14 and later\n\n\n---","ref":"changelog.html#technical-details"},{"type":"extras","title":"Version History - Changelog","doc":"- **1.0.0**: Production release with complete implementation and documentation","ref":"changelog.html#version-history"},{"type":"extras","title":"Contributing - Changelog","doc":"When contributing to this project, please update this changelog with your changes following the format above. Include:\n\n- **Added**: New features\n- **Changed**: Changes in existing functionality\n- **Deprecated**: Soon-to-be removed features\n- **Removed**: Removed features\n- **Fixed**: Bug fixes\n- **Security**: Vulnerability fixes","ref":"changelog.html#contributing"},{"type":"extras","title":"Links - Changelog","doc":"- [Keep a Changelog](https://keepachangelog.com/en/1.0.0/)\n- [Semantic Versioning](https://semver.org/spec/v2.0.0.html)","ref":"changelog.html#links"}],"proglang":"elixir","content_type":"text/markdown","producer":{"name":"ex_doc","version":"0.38.2"}}